@@ -4,7 +4,7 @@
 /*			     Written by Ken Goldman				*/
 /*		       IBM Thomas J. Watson Research Center			*/
 /*										*/
-/* (c) Copyright IBM Corporation 2015 - 2020					*/
+/* (c) Copyright IBM Corporation 2015 - 2023					*/
 /*										*/
 /* All rights reserved.								*/
 /* 										*/
@@ -50,6 +50,7 @@
 #include <ibmtss/tssresponsecode.h>
 #include <ibmtss/tssmarshal.h>
 #include <ibmtss/tsscryptoh.h>
+#include <ibmtss/Unmarshal_fp.h>
 
 #include "objecttemplates.h"
 #include "cryptoutils.h"
@@ -82,6 +83,7 @@ int main(int argc, char *argv[])
     const char			*pemFilename = NULL;
     const char			*ticketFilename = NULL;
     const char			*creationHashFilename = NULL;
+    const char			*creationDataFilename = NULL;
     const char 			*dataFilename = NULL;
     const char			*keyPassword = NULL; 
     const char			*parentPassword = NULL; 
@@ -336,6 +338,16 @@ int main(int argc, char *argv[])
 		printUsage();
 	    }
 	}
+	else if (strcmp(argv[i],"-cd") == 0) {
+	    i++;
+	    if (i < argc) {
+		creationDataFilename = argv[i];
+	    }
+	    else {
+		printf("-cd option needs a value\n");
+		printUsage();
+	    }
+	}
 	else if (strcmp(argv[i],"-pwdk") == 0) {
 	    i++;
 	    if (i < argc) {
@@ -612,7 +624,7 @@ int main(int argc, char *argv[])
 	    sizeInBytes = TSS_GetDigestSize(nalg);
 	    if (out.creationHash.b.size != sizeInBytes) {
 		printf("create: failed, "
-		       "creationData size %u incompatible with name algorithm %04x\n",
+		       "creationHash size %u incompatible with name algorithm %04x\n",
 		       out.creationHash.b.size, nalg);
 		rc = EXIT_FAILURE;
 	    }
@@ -671,6 +683,19 @@ int main(int argc, char *argv[])
 				      out.creationHash.b.size,
 				      creationHashFilename);
     }
+    /* save the optional creation data */
+    if ((rc == 0) && (creationDataFilename != NULL)) {
+	rc = TSS_File_WriteStructure(&out.creationData.creationData,
+				     (MarshalFunction_t)TSS_TPMS_CREATION_DATA_Marshalu,
+				     creationDataFilename);
+    }
+    /* regression test sanity check */
+    if ((rc == 0) && (creationDataFilename != NULL)) {
+	TPMS_CREATION_DATA creationDataTest;
+	rc = TSS_File_ReadStructure(&creationDataTest,
+				    (UnmarshalFunction_t)TSS_TPMS_CREATION_DATA_Unmarshalu,
+				    creationDataFilename);
+    }
     if (rc == 0) {
 	if (tssUtilsVerbose) printf("create: success\n");
     }
@@ -705,10 +730,13 @@ static void printUsage(void)
     printf("\t[-opem\tpublic key PEM format file name (default do not save)]\n");
     printf("\t[-tk\toutput ticket file name (default do not save)]\n");
     printf("\t[-ch\toutput creation hash file name (default do not save)]\n");
+    printf("\t[-cd\toutput creation data file name]\n");
     printf("\n");
     printf("\t-se[0-2] session handle / attributes (default PWAP)\n");
     printf("\t01\tcontinue\n");
     printf("\t20\tcommand decrypt\n");
     printf("\t40\tresponse encrypt\n");
-    exit(1);	
+    printf("\n");
+    printf("Depending on the build configuration, some hash algorithms may not be available.\n");
+    exit(1);
 }
