@@ -4,7 +4,7 @@
 /*			     Written by Ken Goldman				*/
 /*		       IBM Thomas J. Watson Research Center			*/
 /*										*/
-/* (c) Copyright IBM Corporation 2016 - 2020.					*/
+/* (c) Copyright IBM Corporation 2016 - 2022.					*/
 /*										*/
 /* All rights reserved.								*/
 /* 										*/
@@ -61,6 +61,7 @@
 
 #include <openssl/pem.h>
 #include <openssl/x509.h>
+#include <openssl/evp.h>
 
 #include <ibmtss/tssresponsecode.h>
 #include <ibmtss/tssutils.h>
@@ -328,7 +329,7 @@ static const unsigned char iwgPolicyBSha512[] = {
     0x56, 0x08, 0xb4, 0x13, 0xcd, 0x61, 0x6a, 0x5f, 0x6d, 0xb5, 0xb6, 0x07, 0x1a, 0xf9, 0x9b, 0xea,
 };
 
-/* getIwgTemplate() bulds a TPMT_PUBLIC template according to the IWG specification.  It handles
+/* getIwgTemplate() builds a TPMT_PUBLIC template according to the IWG specification.  It handles
    both the low and high NV index ranges.
 */
 
@@ -567,9 +568,12 @@ TPM_RC getIndexX509Certificate(TSS_CONTEXT *tssContext,
    certificate stored in a file.
 
    Returns both the OpenSSL X509 certificate token and RSA public key token.
+
+   For Openssl < 3, rsaKey is an RSA structure.
+   For Openssl 3, rsaKey is an EVP_PKEY,
 */
 
-uint32_t getPubkeyFromDerCertFile(RSA  **rsaPkey,
+uint32_t getPubkeyFromDerCertFile(void **rsaPkey,	/* freed by caller */
 				  X509 **x509,
 				  const char *derCertificateFileName)
 {
@@ -594,7 +598,58 @@ uint32_t getPubkeyFromDerCertFile(RSA  **rsaPkey,
     }
     /* extract the OpenSSL format public key from the X509 token */
     if (rc == 0) {
-	rc = getPubKeyFromX509Cert(rsaPkey, *x509);
+	/* For Openssl < 3, rsaKey is an RSA structure.
+	   For Openssl 3, rsaKey is an EVP_PKEY. */
+	rc = getPubKeyFromX509Cert(rsaPkey, *x509);	/* freed by caller */
+    }
+    /* for debug, print the X509 certificate */
+    if (rc == 0) {
+	if (tssUtilsVerbose) X509_print_fp(stdout, *x509);
+    }
+    if (fp != NULL) {
+	fclose(fp);
+    }
+    return rc;
+}
+
+/* getPubkeyFromDerCertFile3() gets an OpenSSL EVP_PKEY public key token from a DER format X509
+   certificate stored in a file.
+
+   Returns both the OpenSSL X509 certificate token and RSA public key token.
+
+   Differs from getPubkeyFromDerCertFile(), where the public key is version dependent.
+*/
+
+uint32_t getPubkeyFromDerCertFile3(EVP_PKEY **evpPkey,	/* freed by caller */
+				   X509 **x509,	 	/* freed by caller */
+				   const char *derCertificateFileName)
+{
+    uint32_t rc = 0;
+    FILE *fp = NULL;
+
+    /* open the file */
+    if (rc == 0) {
+	fp = fopen(derCertificateFileName, "rb");
+	if (fp == NULL) {
+	    printf("getPubkeyFromDerCertFile3: Error opening %s\n", derCertificateFileName);
+	    rc = TSS_RC_FILE_OPEN;
+	}
+    }
+    /* read the file and convert the X509 DER to OpenSSL format */
+    if (rc == 0) {
+	*x509 = d2i_X509_fp(fp, NULL);
+	if (*x509 == NULL) {
+	    printf("getPubkeyFromDerCertFile3: Error converting %s\n", derCertificateFileName);
+	    rc = TSS_RC_X509_ERROR;
+	}
+    }
+    /* extract the OpenSSL format public key from the X509 token */
+    if (rc == 0) {
+	*evpPkey = X509_get_pubkey(*x509);	/* freed by caller */
+	if (*evpPkey == NULL) {
+	    printf("getPubkeyFromDerCertFile3: X509_get_pubkey failed\n");
+	    rc = TSS_RC_X509_ERROR;
+	}
     }
     /* for debug, print the X509 certificate */
     if (rc == 0) {
@@ -612,9 +667,13 @@ uint32_t getPubkeyFromDerCertFile(RSA  **rsaPkey,
 #ifndef TPM_TSS_NORSA
 
 /* getPubKeyFromX509Cert() gets an OpenSSL RSA public key token from an OpenSSL X509 certificate
-   token. */
+   token.
 
-uint32_t getPubKeyFromX509Cert(RSA  **rsaPkey,
+   For Openssl < 3, rsaKey is an RSA structure.
+   For Openssl 3, rsaKey is an EVP_PKEY,
+*/
+
+uint32_t getPubKeyFromX509Cert(void **rsaPkey,
 			       X509 *x509)
 {
     uint32_t rc = 0;
@@ -623,22 +682,27 @@ uint32_t getPubKeyFromX509Cert(RSA  **rsaPkey,
     if (rc == 0) {
 	evpPkey = X509_get_pubkey(x509);	/* freed @1 */
 	if (evpPkey == NULL) {
-	    printf("getPubKeyFromX509Cert: X509_get_pubkey failed\n");  
+	    printf("getPubKeyFromX509Cert: X509_get_pubkey failed\n");
 	    rc = TSS_RC_X509_ERROR;
 	}
     }
+#if OPENSSL_VERSION_NUMBER < 0x30000000
     if (rc == 0) {
 	*rsaPkey = EVP_PKEY_get1_RSA(evpPkey);
 	if (*rsaPkey == NULL) {
-	    printf("getPubKeyFromX509Cert: EVP_PKEY_get1_RSA failed\n");  
+	    printf("getPubKeyFromX509Cert: EVP_PKEY_get1_RSA failed\n");
 	    rc = TSS_RC_X509_ERROR;
 	}
     }
     if (evpPkey != NULL) {
 	EVP_PKEY_free(evpPkey);		/* @1 */
     }
+#else
+    *rsaPkey = evpPkey;
+#endif
     return rc;
 }
+
 #endif /* TPM_TSS_NORSA */
 
 #ifndef TPM_TSS_NOFILE
@@ -984,7 +1048,7 @@ TPM_RC processEKTemplate(TSS_CONTEXT *tssContext,
 /* processEKCertificate() reads the EK certificate from NV and returns an X509 certificate
    structure.  It also extracts and returns the public modulus.
 
-   The return is void because the structure is opaque to the caller.  This accomodates other crypto
+   The return is void because the structure is opaque to the caller.  This accommodates other crypto
    libraries.
 
    ekCertificate is an X509 structure.
@@ -1005,7 +1069,7 @@ TPM_RC processEKCertificate(TSS_CONTEXT *tssContext,
 				     ekCertificate,	/* freed by caller */
 				     ekCertIndex);
 	if (rc != 0) {
-	    printf("No EK certificate for EK certifcate index %08x\n", ekCertIndex);
+	    printf("No EK certificate for EK certificate index %08x\n", ekCertIndex);
 	}
     }
     /* extract the public modulus from the X509 structure */
@@ -1071,6 +1135,7 @@ TPM_RC convertX509ToDer(uint32_t *certLength,
 
  */
 
+#if OPENSSL_VERSION_NUMBER < 0x30000000
 TPM_RC convertX509ToEc(EC_KEY **ecKey,	/* freed by caller */
 		       X509 *x509)
 {
@@ -1097,6 +1162,7 @@ TPM_RC convertX509ToEc(EC_KEY **ecKey,	/* freed by caller */
     }
     return rc;
 }
+#endif
 
 #endif	/* TPM_TSS_NOECC */
 
@@ -1105,8 +1171,8 @@ TPM_RC convertX509ToEc(EC_KEY **ecKey,	/* freed by caller */
 
    If print is true, prints the EK certificate
 
-   The return is void because the structure is opaque to the caller.  This accomodates other crypto
-   libraries.
+   The ekCertificate return is void because the structure is opaque to the caller.  This
+   accommodates other crypto libraries.
 
    ekCertificate is an X509 structure.
 */
@@ -1131,7 +1197,7 @@ TPM_RC convertCertificatePubKey(uint8_t **modulusBin,	/* freed by caller */
     if (rc == 0) {
 	pkey = X509_get_pubkey(ekCertificate);		/* freed @2 */
 	if (pkey == NULL) {
-#ifndef TPM_TSS_NORSA
+#ifdef TPM_TPM12
 	    if (tssUtilsVerbose) printf("convertCertificatePubKey: "
 				"Could not extract public key from X509 certificate, "
 				"may be TPM 1.2\n");
@@ -1140,12 +1206,10 @@ TPM_RC convertCertificatePubKey(uint8_t **modulusBin,	/* freed by caller */
 	    rc = convertCertificatePubKey12(modulusBin,	/* freed by caller */
 					    modulusBytes,
 					    ekCertificate);
-#else
+#endif	/* TPM_TPM12 */
 	    printf("convertCertificatePubKey: Could not extract X509_PUBKEY public key "
 		   "from X509 certificate\n");
 	    rc =  TPM_RC_INTEGRITY;
-#endif /* TPM_TSS_NORSA */
-
 	}
 	else {
 	    if (rc == 0) {
@@ -1158,7 +1222,11 @@ TPM_RC convertCertificatePubKey(uint8_t **modulusBin,	/* freed by caller */
 	      case EK_CERT_RSA_3072_INDEX_H6:
 	      case EK_CERT_RSA_4096_INDEX_H7:
 		  {
+#if OPENSSL_VERSION_NUMBER < 0x30000000
 		      RSA *rsaKey = NULL;
+#else
+		      EVP_PKEY *rsaKey = NULL;
+#endif
 		      /* check that the public key algorithm matches the ekCertIndex algorithm */
 		      if (rc == 0) {
 			  if (pkeyType != EVP_PKEY_RSA) {
@@ -1169,12 +1237,16 @@ TPM_RC convertCertificatePubKey(uint8_t **modulusBin,	/* freed by caller */
 		      }
 		      /* convert the public key to OpenSSL structure */
 		      if (rc == 0) {
+#if OPENSSL_VERSION_NUMBER < 0x30000000
 			  rsaKey = EVP_PKEY_get1_RSA(pkey);		/* freed @3 */
 			  if (rsaKey == NULL) {
 			      printf("convertCertificatePubKey: Could not extract RSA public key "
 				     "from X509 certificate\n");
 			      rc = TPM_RC_INTEGRITY;
 			  }
+#else		/* use the EVP_PKEY directly */
+			  rsaKey = pkey;
+#endif
 		      }
 		      if (rc == 0) {
 			  rc = convertRsaKeyToPublicKeyBin(modulusBytes,
@@ -1185,7 +1257,9 @@ TPM_RC convertCertificatePubKey(uint8_t **modulusBin,	/* freed by caller */
 			  if (print) TSS_PrintAll("Certificate public key:",
 						  *modulusBin, *modulusBytes);
 		      }    
+#if OPENSSL_VERSION_NUMBER < 0x30000000
 		      RSA_free(rsaKey);   		/* @3 */
+#endif
 		  }
 		  break;
 #endif /* TPM_TSS_NORSA */
@@ -1197,7 +1271,11 @@ TPM_RC convertCertificatePubKey(uint8_t **modulusBin,	/* freed by caller */
 	      case EK_CERT_ECC_NISTP521_INDEX_H4:
 	      case EK_CERT_ECC_SM2P256INDEX_H5:
 		  {
+#if OPENSSL_VERSION_NUMBER < 0x30000000
 		      EC_KEY *ecKey = NULL;
+#else
+		      EVP_PKEY *ecKey = NULL;
+#endif
 		      /* check that the public key algorithm matches the ekCertIndex algorithm */
 		      if (rc == 0) {
 			  if (pkeyType != EVP_PKEY_EC) {
@@ -1208,12 +1286,16 @@ TPM_RC convertCertificatePubKey(uint8_t **modulusBin,	/* freed by caller */
 		      }
 		      /* convert the public key to OpenSSL structure */
 		      if (rc == 0) {
+#if OPENSSL_VERSION_NUMBER < 0x30000000
 			  ecKey = EVP_PKEY_get1_EC_KEY(pkey);		/* freed @3 */
 			  if (ecKey == NULL) {
 			      printf("convertCertificatePubKey: Could not extract EC public key "
 				     "from X509 certificate\n");
 			      rc = TPM_RC_INTEGRITY;
 			  }
+#else		/* use the EVP_PKEY directly */
+			  ecKey = pkey;
+#endif
 		      }
 		      if (rc == 0) {
 			  rc = convertEcKeyToPublicKeyBin(modulusBytes,
@@ -1224,7 +1306,9 @@ TPM_RC convertCertificatePubKey(uint8_t **modulusBin,	/* freed by caller */
 			  if (print) TSS_PrintAll("Certificate public key:",
 						  *modulusBin, *modulusBytes);
 		      }
+#if OPENSSL_VERSION_NUMBER < 0x30000000
 		      EC_KEY_free(ecKey);   		/* @3 */
+#endif
 		  }
 		  break;
 #endif	/* TPM_TSS_NOECC */
@@ -1241,20 +1325,25 @@ TPM_RC convertCertificatePubKey(uint8_t **modulusBin,	/* freed by caller */
     return rc;
 }
 
-#ifndef TPM_TSS_NORSA
+#ifdef TPM_TPM12
 
 TPM_RC convertCertificatePubKey12(uint8_t **modulusBin,	/* freed by caller */
 				  int *modulusBytes,
 				  X509 *ekCertificate)
 {
     TPM_RC		rc = 0;
+#ifndef TPM_TSS_NORSA
     int			irc;
     X509_PUBKEY 	*pubkey = NULL;
     ASN1_OBJECT 	*ppkalg = NULL;			/* ignore OID */
     const unsigned char *pk = NULL;			/* do not free */
     int 		ppklen;
     X509_ALGOR 		*palg = NULL;			/* algorithm identifier for public key */
+#if OPENSSL_VERSION_NUMBER < 0x30000000
     RSA 		*rsaKey = NULL;
+#else
+    EVP_PKEY 		*rsaKey = NULL;
+#endif
 
     /* get internal pointer to the public key in the certificate */
     if (rc == 0) {
@@ -1278,7 +1367,12 @@ TPM_RC convertCertificatePubKey12(uint8_t **modulusBin,	/* freed by caller */
     }
     if (rc == 0) {
 	const unsigned char *tmppk = pk;	/* because d2i moves the pointer */
+#if OPENSSL_VERSION_NUMBER < 0x30000000
 	rsaKey = d2i_RSAPublicKey(NULL, &tmppk, ppklen);	/* freed @1 */
+#else
+	rsaKey = d2i_PublicKey(EVP_PKEY_RSA, NULL,
+			       &tmppk, (long)ppklen);
+#endif
 	if (rsaKey == NULL) {
 	    printf("convertCertificatePubKey12: Could not convert to RSA structure\n");
 	    rc = TPM_RC_INTEGRITY;
@@ -1290,13 +1384,14 @@ TPM_RC convertCertificatePubKey12(uint8_t **modulusBin,	/* freed by caller */
 					 rsaKey);
 	TSS_PrintAll("convertCertificatePubKey12", *modulusBin, *modulusBytes);
     }
-    if (rsaKey != NULL) {
-	RSA_free(rsaKey);		/* @1 */
-    }
+    TSS_RsaFree(rsaKey);		/* @1 */
+#else  /* TPM_TSS_NORSA */
+    rc = TPM_RC_INTEGRITY;
+#endif /* TPM_TSS_NORSA */
     return rc;
 }
 
-#endif /* TPM_TSS_NORSA */
+#endif /* TPM_TPM12 */
 
 #ifndef TPM_TSS_NOFILE		/* stdout is a file descriptor */
 
@@ -1373,7 +1468,7 @@ uint32_t convertPemToX509(X509 **x509,				/* freed by caller */
 
 /* convertDerToX509() converts a DER stream to an OpenSSL X509 structure
 
-   The return is void because the structure is opaque to the caller.  This accomodates other crypto
+   The return is void because the structure is opaque to the caller.  This accommodates other crypto
    libraries.
 */
 
@@ -1391,9 +1486,9 @@ uint32_t convertDerToX509(void **x509Certificate,			/* freed by caller */
     return rc;
 }
 
-/* x509FreeStructure() is the library specific free structure.
+/* x509Free Structure() is the library specific free structure.
 
-   The parameter is void because the structure is opaque to the caller.  This accomodates other
+   The parameter is void because the structure is opaque to the caller.  This accommodates other
    crypto libraries.
 */
 
@@ -1407,7 +1502,7 @@ void x509FreeStructure(void *x509)
 
 /* x509PrintStructure() prints the structure to stdout
 
-   The parameter is void because the structure is opaque to the caller.  This accomodates other
+   The parameter is void because the structure is opaque to the caller.  This accommodates other
    crypto libraries.
 */
 
@@ -1472,7 +1567,7 @@ uint32_t convertPemMemToX509(X509 **x509,		/* freed by caller */
 
 /* convertX509ToPem() writes an OpenSSL X509 structure to a PEM format file
 
-   The return is void because the structure is opaque to the caller.  This accomodates other crypto
+   The return is void because the structure is opaque to the caller.  This accommodates other crypto
    libraries.
  
    For OpenSSL, the type is X509*
@@ -1835,7 +1930,7 @@ TPM_RC startCertificate(X509 *x509Certificate,	/* X509 certificate to be generat
     ASN1_TIME 		*arc;			/* return code */
     ASN1_INTEGER 	*x509Serial;		/* certificate serial number in ASN1 */
     BIGNUM 		*x509SerialBN;		/* certificate serial number as a BIGNUM */
-    unsigned char 	x509Serialbin[SHA1_DIGEST_SIZE]; /* certificate serial number in binary */
+    unsigned char 	x509Serialbin[EVP_MAX_MD_SIZE]; /* certificate serial number in binary */
     X509_NAME 		*x509IssuerName;	/* composite issuer name, key/value pairs */
     X509_NAME 		*x509SubjectName;	/* composite subject name, key/value pairs */
 
@@ -1855,11 +1950,20 @@ TPM_RC startCertificate(X509 *x509Certificate,	/* X509 certificate to be generat
       add certificate serial number
     */
     if (rc == 0) {
+	const EVP_MD *type;
+
 	if (tssUtilsVerbose) printf("startCertificate: Adding certificate serial number\n");
 	/* to create a unique serial number, hash the key to be certified */
-	SHA1(keyBuffer, keyLength, x509Serialbin);
-	/* convert the SHA1 digest to a BIGNUM */
-	x509SerialBN = BN_bin2bn(x509Serialbin, SHA1_DIGEST_SIZE, x509SerialBN);
+	type = EVP_sha256();
+	irc = EVP_Digest(keyBuffer, keyLength, x509Serialbin, NULL, type, NULL);
+	if (irc == 0) {
+	    printf("startCertificate: Error in serial number EVP_Digest\n");
+	    rc = TSS_RC_X509_ERROR;
+	}
+    }
+    if (rc == 0) {
+	/* convert the digest to a BIGNUM, use 20 octets */
+	x509SerialBN = BN_bin2bn(x509Serialbin, 20, x509SerialBN);
 	if (x509SerialBN == NULL) {
 	    printf("startCertificate: Error in serial number BN_bin2bn\n");
 	    rc = TSS_RC_X509_ERROR;
@@ -2090,7 +2194,7 @@ TPM_RC addCertKeyEccT(X509 *x509Certificate,
     if (rc == 0) {
 	irc = X509_set_pubkey(x509Certificate, evpPubkey);
 	if (irc != 1) {
-	    printf("addCertKeyEcc: Error adding public key to certificate\n");
+	    printf("addCertKeyEccT: Error adding public key to certificate\n");
 	    rc = TSS_RC_X509_ERROR;
 	}
     }
@@ -2101,6 +2205,8 @@ TPM_RC addCertKeyEccT(X509 *x509Certificate,
     return rc;
 }
 
+#if OPENSSL_VERSION_NUMBER < 0x30000000
+
 /* addCertKeyEcc() adds the TPM ECC public key (the key to be certified) to the openssl X509
    certificate.
 
@@ -2134,6 +2240,7 @@ TPM_RC addCertKeyEcc(X509 *x509Certificate,
     }
     return rc;
 }
+#endif
 
 #endif	/* TPM_TSS_NOECC */
 #endif /* TPM_TPM20 */
@@ -2479,7 +2586,7 @@ TPM_RC processValidatePrimary(uint8_t *publicKeyBin,		/* from certificate */
 		    != publicKeyBytes) {
 		    printf("processValidatePrimary: "
 			   "X509 certificate key length %u does not match "
-			   "output of createprimary x %u +y %u\n",
+			   "output of createprimary x %u + y %u\n",
 			   publicKeyBytes,
 			   tpmtPublic->unique.ecc.x.t.size,
 			   tpmtPublic->unique.ecc.y.t.size);
@@ -2514,7 +2621,7 @@ TPM_RC processValidatePrimary(uint8_t *publicKeyBin,		/* from certificate */
 	else
 #endif /* TPM_TSS_NOECC */
 	    {
-		printf("processValidatePrimary=: "
+		printf("processValidatePrimary: "
 		       "ekCertIndex %08x (asymmetric algorithm) not supported\n", ekCertIndex);
 		rc = TPM_RC_INTEGRITY;
 	    }
@@ -2526,38 +2633,38 @@ TPM_RC processValidatePrimary(uint8_t *publicKeyBin,		/* from certificate */
 }
 
 /* processPrimary() is deprecated.  It is missing the endorsement auth.  It is missing the key
-   password, new for the high range EKs.
+   password, new for the high range EKs.  It always validates the low range EK public keys.
 
+   See processPrimaryEN()
 */
 
 TPM_RC processPrimary(TSS_CONTEXT *tssContext,
-		      TPM_HANDLE *keyHandle,		/* primary key handle */
+		      TPM_HANDLE *keyHandle,
 		      TPMI_RH_NV_INDEX ekCertIndex,
 		      TPMI_RH_NV_INDEX ekNonceIndex, 
 		      TPMI_RH_NV_INDEX ekTemplateIndex,
-		      unsigned int noFlush,		/* TRUE - don't flush the primary key */
+		      unsigned int noFlush,
 		      int print)
 {
-    TPM_RC rc = processPrimaryE(tssContext,
-				keyHandle,
-				NULL,	/* endorsement auth */
-				NULL,	/* endorsement key password */
-				ekCertIndex,
-				ekNonceIndex,
-				ekTemplateIndex,
-				noFlush,
-				print);
+    TPM_RC rc = processPrimaryEN(tssContext,
+				 keyHandle,
+				 NULL,	/* default endorsement auth */
+				 NULL,	/* default endorsement key password */
+				 ekCertIndex,
+				 ekNonceIndex,
+				 ekTemplateIndex,
+				 noFlush,
+				 FALSE,		/* default noPub */
+				 print);
     return rc;
 }
 
-/* processPrimaryE() reads the EK nonce and EK template from NV.  It combines them to form the
-   createprimary input.  It creates the primary key.
+/* processPrimaryE() is deprecated.  It always validates the low range EK public keys.
 
-   It reads the EK certificate from NV.  It extracts the public key.
-
-   Finally, it compares the public key in the certificate to the public key output of createprimary.
+   See processPrimaryEN()
 */
 
+
 TPM_RC processPrimaryE(TSS_CONTEXT *tssContext,
 		       TPM_HANDLE *keyHandle,		/* primary key handle */
 		       const char *endorsementPassword,
@@ -2565,8 +2672,59 @@ TPM_RC processPrimaryE(TSS_CONTEXT *tssContext,
 		       TPMI_RH_NV_INDEX ekCertIndex,
 		       TPMI_RH_NV_INDEX ekNonceIndex,
 		       TPMI_RH_NV_INDEX ekTemplateIndex,
-		       unsigned int noFlush,		/* TRUE - don't flush the primary key */
+		       unsigned int noFlush,
 		       int print)
+{
+    TPM_RC rc = 0;
+    unsigned int noPub = 1;
+
+    /* the backward compatible behavior was to always validate the low range public key
+       and never validate the high range public key */
+#ifndef TPM_TSS_NORSA
+    if (ekCertIndex == EK_CERT_RSA_INDEX) {
+	noPub = 0;
+    }
+#endif /* TPM_TSS_NORSA */
+#ifndef TPM_TSS_NOECC
+   if (ekCertIndex == EK_CERT_EC_INDEX) {
+	noPub = 0;
+    }
+#endif	/* TPM_TSS_NOECC */
+    rc = processPrimaryEN(tssContext,
+			  keyHandle,		/* primary key handle */
+			  endorsementPassword,
+			  keyPassword,
+			  ekCertIndex,
+			  ekNonceIndex,
+			  ekTemplateIndex,
+			  noFlush,
+			  noPub,		/* noPub */
+			  print);
+    return rc;
+}
+
+/* processPrimaryEN() reads the EK nonce and EK template from NV if present.  It forms the
+   createprimary input.  It creates the primary key.
+
+   It reads the EK certificate from NV.  It extracts the public key.
+
+   If noPub is false, it compares the public key in the certificate to the public key output of
+   createprimary.
+
+   If noFlush is false, the EK is flushed after processing.  If noFlush is true, the EK remains in
+   the TPM as a transient key.
+*/
+
+TPM_RC processPrimaryEN(TSS_CONTEXT *tssContext,
+			TPM_HANDLE *keyHandle,		/* primary key handle */
+			const char *endorsementPassword,
+			const char *keyPassword,
+			TPMI_RH_NV_INDEX ekCertIndex,
+			TPMI_RH_NV_INDEX ekNonceIndex,
+			TPMI_RH_NV_INDEX ekTemplateIndex,
+			unsigned int noFlush,		/* TRUE - don't flush the primary key */
+			unsigned int noPub,		/* TRUE - don't verify the pubic key */
+			int print)
 {
     TPM_RC			rc = 0;
     void 			*ekCertificate = NULL;
@@ -2576,13 +2734,12 @@ TPM_RC processPrimaryE(TSS_CONTEXT *tssContext,
     TPMT_PUBLIC 		tpmtPublicOut;		/* primary key */
     uint8_t 			*publicKeyBin = NULL;	/* from certificate */
     int				publicKeyBytes;
-    int 			validate = FALSE;	/* validate the certificate */
 
     /* get the EK nonce */
     if (rc == 0) {
 	rc = processEKNonce(tssContext, &nonce, &nonceSize, ekNonceIndex, print); /* freed @1 */
 	if ((rc & 0xff) == TPM_RC_HANDLE) {
-	    if (print) printf("processPrimary: EK nonce not found, use default template\n");
+	    if (print) printf("processPrimaryEN: EK nonce not found, use default template\n");
 	    rc = 0;
 	}
     }
@@ -2605,21 +2762,8 @@ TPM_RC processPrimaryE(TSS_CONTEXT *tssContext,
 				   noFlush,
 				   print);
     }
-    /* validate against the certificate if the algorithm is compiled in */
-    if (rc == 0) {
-#ifndef TPM_TSS_NORSA
-	if (ekCertIndex == EK_CERT_RSA_INDEX) {
-	    validate = TRUE;
-	}
-#endif /* TPM_TSS_NORSA */
-#ifndef TPM_TSS_NOECC
-	if (ekCertIndex == EK_CERT_EC_INDEX) {
-	    validate = TRUE;
-	}
-#endif	/* TPM_TSS_NOECC */
-    }
     /* get the EK certificate */
-    if ((rc == 0) && validate) {
+    if ((rc == 0) && !noPub) {
 	rc = processEKCertificate(tssContext,
 				  &ekCertificate,			/* freed @2 */
 				  &publicKeyBin, &publicKeyBytes,	/* freed @3 */
@@ -2627,14 +2771,14 @@ TPM_RC processPrimaryE(TSS_CONTEXT *tssContext,
 				  print);
     }
     /* compare the public key in the EK certificate to the public key output */
-    if ((rc == 0) && validate) {
+    if ((rc == 0) && !noPub) {
 	rc = processValidatePrimary(publicKeyBin,	/* certificate */
 				    publicKeyBytes,
 				    &tpmtPublicOut,	/* primary key */
 				    ekCertIndex,
 				    print);
     }
-    if ((rc == 0) && validate) {
+    if ((rc == 0) && !noPub) {
 	if (print) printf("Public key from X509 certificate matches output of createprimary\n");
     } 
     free(nonce);			/* @1 */
