@@ -54,7 +54,7 @@
 /*    arising in any way out of use or reliance upon this specification or any 	*/
 /*    information herein.							*/
 /*										*/
-/*  (c) Copyright IBM Corp. and others, 2012 - 2020				*/
+/*  (c) Copyright IBM Corp. and others, 2012 - 2022				*/
 /*										*/
 /********************************************************************************/
 
@@ -79,10 +79,20 @@
 // From Vendor-Specific: Table 4 - Defines for Key Size Constants
 
 #ifdef TPM_TSS_SO_0
-#define  MAX_RSA_KEY_BYTES          256
+#define  MAX_RSA_KEY_BYTES          256		/* TSS 0.x */
 #else
-#define  MAX_RSA_KEY_BYTES          512
-#endif
+
+#ifdef TPM_TSS_SO_1
+#define  MAX_RSA_KEY_BYTES          512		/* TSS 1.x */
+#else
+
+#define  MAX_RSA_KEY_BYTES          2048	/* TSS 2.x and up */
+#endif	/* TPM_TSS_SO_1 */
+#endif	/* TPM_TSS_SO_0 */
+
+#define RSA_MAX_PRIME           	(MAX_RSA_KEY_BYTES / 2)
+#define RSA_PRIVATE_SIZE        	(RSA_MAX_PRIME * 5)
+#define PRIVATE_VENDOR_SPECIFIC_BYTES   RSA_PRIVATE_SIZE
 
 #ifdef TPM_TSS_SO_0
 #define  MAX_ECC_KEY_BYTES     48
@@ -90,7 +100,9 @@
 #define  MAX_ECC_KEY_BYTES     128
 #endif
 
-/* This is the PC Client minimum value, and should be used for applications. */
+/* 24 is the PC Client standard value, and should be used for applications. If this value is
+   changed, utilities must be recompiled.  If it is changed beyond 64, PCR_SELECT_MAX must change,
+   which will be an ABI break. */
 #define IMPLEMENTATION_PCR		24
 
 #define MAX_HANDLE_NUM			3	/* the maximum number of handles in the handle
@@ -102,7 +114,8 @@
 #ifdef TPM_TSS_SO_0
 #define PCR_SELECT_MAX			((IMPLEMENTATION_PCR+7)/8)
 #else
-/* increased to 8 to handle up to 64 PCRs */
+/* increased to 8 to handle up to 64 PCRs.  If IMPLEMENTATION_PCR is increased beyond 64, this value
+   must change, which will be an ABI break. */
 #define PCR_SELECT_MAX			8
 #endif
 
@@ -219,7 +232,7 @@
 #define  ALG_ECMQV             (ALG_NO*ALG_ECC)
 #define  ALG_SYMCIPHER         ALG_YES
 #define  ALG_KDF1_SP800_56A    (ALG_YES*ALG_ECC)
-#define  ALG_KDF2              ALG_NO
+#define  ALG_KDF2              ALG_YES
 #define  ALG_KDF1_SP800_108    ALG_YES
 #define  ALG_CTR               ALG_YES
 #define  ALG_OFB               ALG_YES
@@ -247,6 +260,8 @@
 #define  CC_DictionaryAttackLockReset     CC_YES
 #define  CC_DictionaryAttackParameters    CC_YES
 #define  CC_Duplicate                     CC_YES
+#define  CC_ECC_Encrypt                   (CC_YES*ALG_ECC)
+#define  CC_ECC_Decrypt                   (CC_YES*ALG_ECC)
 #define  CC_ECC_Parameters                (CC_YES*ALG_ECC)
 #define  CC_ECDH_KeyGen                   (CC_YES*ALG_ECC)
 #define  CC_ECDH_ZGen                     (CC_YES*ALG_ECC)
@@ -1265,9 +1280,11 @@ typedef  UINT32             TPM_CC;
 #if CC_CertifyX509 == YES
 #define  TPM_CC_CertifyX509 		      (TPM_CC)(0x00000197)
 #endif
+#define TPM_CC_ECC_Encrypt		      (TPM_CC)(0x00000199)
+#define TPM_CC_ECC_Decrypt		      (TPM_CC)(0x0000019A)
 
 /* Compile variable. May increase based on implementation. */
-#define  TPM_CC_LAST			      (TPM_CC)(0x00000197)
+#define  TPM_CC_LAST			      (TPM_CC)(0x0000019a)
 
 #ifndef CC_Vendor_TCG_Test
 #   define CC_Vendor_TCG_Test NO
@@ -1423,6 +1440,8 @@ typedef  UINT32             TPM_CC;
 					  + (ADD_FILL || CC_EncryptDecrypt2)            /* 0x00000193 */ \
 					  + (ADD_FILL || CC_PolicyNvWritten)            /* 0x0000018f */ \
 					  + (ADD_FILL || CC_CertifyX509)                /* 0x00000197 */ \
+					  + (ADD_FILL || CC_ECC_Encrypt)                /* 0x00000199 */ \
+					  + (ADD_FILL || CC_ECC_Decrypt)                /* 0x0000019a */ \
 					  )
 #define VENDOR_COMMAND_ARRAY_SIZE   ( 0				\
 				      + CC_Vendor_TCG_Test	\
